#!/usr/bin/env python3

import gi
gi.require_version('Gtk', '3.0')
gi.require_version('GdkPixbuf', '2.0')
gi.require_version('Poppler', '0.18')
gi.require_version('GExiv2', '0.10')
from gi.repository import Gtk, Gio, GLib

from comipoli_window import ComipoliWindow
from comipoli_shortcuts import ComipoliShortcuts
from comipoli_preference import ComipoliPreference

class ComipoliApplication(Gtk.Application):
    def __init__(self, version):
        GLib.set_prgname('Comipoli')
        self.version = version
        Gtk.Application.__init__(
            self,
            application_id='org.sasakima.comipoli',
            #flags=Gio.ApplicationFlags.HANDLES_COMMAND_LINE | Gio.ApplicationFlags.HANDLES_OPEN)
            flags=Gio.ApplicationFlags.HANDLES_COMMAND_LINE)
        # option
        self.add_main_option('version', b'v', GLib.OptionFlags.NONE, GLib.OptionArg.NONE, 'Show Comipoli Version', None)
        self.add_main_option('new-window', b'n', GLib.OptionFlags.NONE, GLib.OptionArg.NONE, 'New Comipoli Window', None)

    def do_command_line(self, command_line):
        # search option
        options = command_line.get_options_dict()
        if options.lookup_value('new-window', GLib.VariantType.new('b')):
            ComipoliWindow(self)
        # get filename
        arg = command_line.get_arguments()[1:]
        for s in arg:
            if not s.startswith('-'):
                if '//' in s:
                    f = Gio.file_new_for_uri(s)
                else:
                    f = Gio.file_new_for_path(s)
                if f.query_exists():
                    self.props.active_window.set_uri(f.get_uri())
                    break
        return 0

    def do_handle_local_options(self, options):
        # get option
        if options.lookup_value('version', GLib.VariantType.new('b')):
            print(f'comipoli {self.version}')
            return 0
        return -1

    def do_shutdown(self):
        Gtk.Application.do_shutdown(self)
        settings = Gio.Settings(schema='org.sasakima.comipoli')
        settings.set_int('thumbnail-height', self.thumbnail_height)
        settings.set_boolean('esc-to-exit', self.esc_exit)
        settings.set_boolean('enable-pdf', self.is_pdf)
        settings.set_int('cursor-change-pixel', self.cursor_change_pixel)

    def do_startup(self):
        Gtk.Application.do_startup(self)
        # Dark Theme
        settings = Gtk.Settings.get_default()
        settings.props.gtk_application_prefer_dark_theme = True
        # Search Archiver
        self.is_unrar = GLib.spawn_command_line_sync('type unrar')[3] == 0
        self.is_7za = GLib.spawn_command_line_sync('type 7za')[3] == 0
        # Read GSettings
        settings = Gio.Settings(schema='org.sasakima.comipoli')
        w, h = settings.get_value('window-size').unpack()
        self.win_width = w
        self.win_height = h
        self.left_to_right = settings.get_boolean('left-to-right')
        self.thumbnail_height = settings.get_int('thumbnail-height')
        self.esc_exit = settings.get_boolean('esc-to-exit')
        self.is_pdf = settings.get_boolean('enable-pdf')
        self.cursor_change_pixel = settings.get_int('cursor-change-pixel')
        # Accel
        self.set_accels_for_action('app.new_file_action', ['<Control>O'])
        self.set_accels_for_action('app.new_window_action', ['<Control>N'])
        self.set_accels_for_action('app.shortcut_action', ['<Control>F1', 'question'])
        self.set_accels_for_action('app.quit_action', ['<Control>Q'])
        # Action
        new_file_action   = Gio.SimpleAction(name='new_file_action')
        new_window_action = Gio.SimpleAction(name='new_window_action')
        preference_action = Gio.SimpleAction(name='preference_action')
        shortcut_action   = Gio.SimpleAction(name='shortcut_action')
        about_action      = Gio.SimpleAction(name='about_action')
        quit_action       = Gio.SimpleAction(name='quit_action')
        self.add_action(new_file_action)
        self.add_action(new_window_action)
        self.add_action(preference_action)
        self.add_action(shortcut_action)
        self.add_action(about_action)
        self.add_action(quit_action)
        # connect
        new_file_action.connect('activate', self.on_new_file_action)
        new_window_action.connect('activate', self.on_new_window_action)
        preference_action.connect('activate', self.on_preference_action)
        shortcut_action.connect('activate', self.on_shortcut_action)
        about_action.connect('activate', self.on_about_action)
        quit_action.connect('activate', self.on_quit_action)
        # Create Window
        ComipoliWindow(self)

    def on_new_file_action(self, action, parameter):
        self.props.active_window.open_dialog()

    def on_new_window_action(self, action, parameter):
        ComipoliWindow(self)

    def on_preference_action(self, action, parameter):
        dlg = ComipoliPreference(self.props.active_window)
        dlg.run()
        dlg.destroy()

    def on_shortcut_action(self, action, parameter):
        shortcuts = ComipoliShortcuts()
        self.props.active_window.set_help_overlay(shortcuts)
        shortcuts.connect('hide', self.on_shortcuts_hide)
        shortcuts.show_all()

    def on_about_action(self, action, parameter):
        d = Gtk.AboutDialog(
            authors=['sasakima-nao <sasakimanao@gmail.com>'],
            comments='Comic book Archive (CBZ|CBR|CB7) and PDF Viewer',
            copyright='Copyright(C) 2016-2020 sasakima-nao',
            license_type=Gtk.License.GPL_2_0,
            logo_icon_name='comipoli',
            program_name='Comipoli',
            version=self.version,
            website='http://palepoli.skr.jp/',
            transient_for=self.props.active_window)
        d.run()
        d.destroy()

    def on_quit_action(self, action, parameter):
        self.quit()

    def on_shortcuts_hide(self, widget):
        widget.destroy()

'''
    def do_open(self, files, n_file, hint):
        uri = files[0].get_uri()
        self.props.active_window.set_uri(uri)
        self.props.active_window.present()

    def do_activate(self):
        self.props.active_window.present()
'''
