#!/usr/bin/env python3

from gi.repository import GLib, Gio, GdkPixbuf, Poppler, GExiv2
import os, re, subprocess

PICEXT = '\.(jpe?g|png|gif)$'

class ComipoliArchive:
    def __init__(self):
        self.status = 0
        self.namelist = []
        self.path = ''
        self.pdf = None
        self.max = 0

    def new_archive(self, uri, app):
        arc = Gio.file_new_for_uri(uri)
        self.path = arc.get_path()
        if re.search(r'\.(cbz|zip)$', self.path, re.I):
            self.status = 0
            self.pdf = None
            self.namelist.clear()
            sp = Gio.Subprocess.new(['zipinfo', '-1', self.path], Gio.SubprocessFlags.STDOUT_PIPE)
            istream = sp.get_stdout_pipe()
            dstream = Gio.DataInputStream(base_stream=istream)
            while True:
                line = dstream.read_line_utf8()[0]
                if line == None:
                    break
                if re.search(PICEXT, line, re.I):
                    self.namelist.append(line)
            self.namelist.sort()
            self.max = len(self.namelist)
        elif app.is_unrar and re.search(r'\.(cbr|rar)$', self.path, re.I):
            self.status = 1
            self.pdf = None
            self.namelist.clear()
            sp = Gio.Subprocess.new(['unrar', 'vt', '-p-', '--', self.path], Gio.SubprocessFlags.STDOUT_PIPE)
            istream = sp.get_stdout_pipe()
            dstream = Gio.DataInputStream(base_stream=istream)
            while True:
                line = dstream.read_line_utf8()[0]
                if line == None:
                    break
                s = line.lstrip()
                if s.startswith('Name: '):
                    name = s[6:]
                    if re.search(PICEXT, name, re.I):
                        self.namelist.append(name)
            self.namelist.sort()
            self.max = len(self.namelist)
        elif app.is_7za and re.search(r'\.(cb7|7z)$', self.path, re.I):
            self.status = 2
            self.pdf = None
            self.namelist.clear()
            sp = Gio.Subprocess.new(['7za', 'l', '-slt', self.path], Gio.SubprocessFlags.STDOUT_PIPE)
            istream = sp.get_stdout_pipe()
            dstream = Gio.DataInputStream(base_stream=istream)
            while True:
                line = dstream.read_line_utf8()[0]
                if line == None:
                    break
                if line.startswith('Path'):
                    name = line[7:]
                    if re.search(PICEXT, name, re.I):
                        self.namelist.append(name)
            self.namelist.sort()
            self.max = len(self.namelist)
        elif app.is_pdf and re.search(r'\.pdf$', self.path, re.I):
            self.status = 3
            self.namelist.clear()
            f = Gio.File.new_for_path(self.path)
            self.pdf = Poppler.Document.new_from_gfile(f)
            self.max = self.pdf.get_n_pages()
        else:
            return False
        return True

    def _zip_escape(self, filename):
        ESCAPE = '[]*?!^-\\'
        res = ''
        for s in filename:
            if s in ESCAPE:
                res += '\\'
            res += s
        return res

    def _on_wait(self, proc, res):
        proc.wait_check_finish(res)

    def __getitem__(self, num):
        if num == self.max:
            raise IndexError
        try:
            if (self.status == 1):
                args = ['unrar', 'p', '-inul', '-@', '--', self.path, self.namelist[num]]
            elif (self.status == 2):
                args = ['7za', 'x', '-so', self.path, self.namelist[num]]
            elif (self.status == 3):
                return self.pdf.get_page(num)
            else:
                args = ['unzip', '-pj', self.path, self._zip_escape(self.namelist[num])]
            # tag
            ori = 0
            pr = subprocess.Popen(args, encoding='UTF-8', stdout=subprocess.PIPE, stderr=subprocess.PIPE)
            fd = pr.stdout.fileno()
            with open(fd, 'rb') as f:
                metadata = GExiv2.Metadata()
                metadata.open_buf(f.read())
                ori = metadata.get_orientation()
            pr.wait()
            # pixbuf
            sp = Gio.Subprocess.new(args, Gio.SubprocessFlags.STDOUT_PIPE | Gio.SubprocessFlags.STDERR_MERGE)
            sp.wait_check_async(None, self._on_wait)
            stream = sp.get_stdout_pipe()
            p = GdkPixbuf.Pixbuf.new_from_stream(stream)
            if ori == GExiv2.Orientation.HFLIP:
                p = p.flip(True)
            elif ori == GExiv2.Orientation.ROT_180:
                p = p.rotate_simple(GdkPixbuf.PixbufRotation.UPSIDEDOWN)
            elif ori == GExiv2.Orientation.VFLIP:
                p = p.flip(False)
            elif ori == GExiv2.Orientation.ROT_90_HFLIP:
                p = p.rotate_simple(GdkPixbuf.PixbufRotation.CLOCKWISE).flip(True)
            elif ori == GExiv2.Orientation.ROT_90:
                p = p.rotate_simple(GdkPixbuf.PixbufRotation.CLOCKWISE)
            elif ori == GExiv2.Orientation.ROT_90_VFLIP:
                p = p.rotate_simple(GdkPixbuf.PixbufRotation.CLOCKWISE).flip(False)
            elif ori == GExiv2.Orientation.ROT_270:
                p = p.rotate_simple(GdkPixbuf.PixbufRotation.COUNTERCLOCKWISE)
            stream.close()
            sp.force_exit()
            return p
        except Exception as e:
            raise e

    def __len__(self):
        return self.max
