#!/usr/bin/env python3

from gi.repository import Gtk, Gio, GLib, Gdk, GdkPixbuf, GObject

import comipoli_numsort, html, cairo
from comipoli_thumbnail import ComipoliThumbnail
from comipoli_headerbar import ComipoliHeaderBar
from comipoli_archive import ComipoliArchive
from comipoli_canvas import ComipoliCanvas
from comipoli_upperbar import ComipoliUpperBar
from comipoli_message import ComipoliMessage
from comipoli_menubutton import ComipoliMenuButton
from comipoli_nextdialog import ComipolNextDialog

class ComipoliWindow(Gtk.ApplicationWindow):
    def __init__(self, app):
        Gtk.ApplicationWindow.__init__(self, application=app)
        # var
        self.app = app
        self.thumbnail_height = 120
        self.num = 0
        self.archive = ComipoliArchive()
        self.is_fullscreen = False
        self.long_keypress = 0
        self.long_click = 0
        self.cursor_id = 0
        self.thumbnail_array = []
        # Cursor
        display = Gdk.Display.get_default()
        self.cursor_left = Gdk.Cursor.new_for_display(display , Gdk.CursorType.SB_LEFT_ARROW)
        self.cursor_right = Gdk.Cursor.new_for_display(display , Gdk.CursorType.SB_RIGHT_ARROW)
        self.cursor_blank = Gdk.Cursor.new_for_display(display , Gdk.CursorType.BLANK_CURSOR)
        # Icon
        self.set_icon_name('comipoli')
        # Canvas
        self.canvas = ComipoliCanvas()
        # HeaderBar
        self.headerbar = ComipoliHeaderBar()
        self.headerbar.lr_button.connect('toggled', self.on_lr_button_toggled)
        self.headerbar.grid_button.connect('clicked', self.on_grid_button_clicked)
        self.set_titlebar(self.headerbar)
        # UpperBar
        self.upperbar = ComipoliUpperBar()
        self.upperbar.grid_button.connect('clicked', self.on_grid_button_clicked)
        self.upperbar.restore_button.connect('clicked', self.on_restore_button_clicked)
        GObject.Binding.bind_property(
            self.headerbar.lr_button, 'active',
            self.upperbar.lr_button, 'active',
            GObject.BindingFlags.BIDIRECTIONAL)
        GObject.Binding.bind_property(
            self.headerbar.grid_button, 'sensitive',
            self.upperbar.grid_button, 'sensitive',
            GObject.BindingFlags.BIDIRECTIONAL)
        # menu
        self.menu = ComipoliMenuButton()
        self.headerbar.pack_end(self.menu)
        self.menuf = ComipoliMenuButton()
        self.upperbar.pack_end(self.menuf)
        # pack
        overlay = Gtk.Overlay()
        overlay.add(self.canvas)
        overlay.add_overlay(self.upperbar)
        self.add(overlay)
        # DnD
        self.drag_dest_add_uri_targets()
        # signal
        self.canvas.set_events(Gdk.EventMask.BUTTON_PRESS_MASK | Gdk.EventMask.STRUCTURE_MASK )
        self.canvas.connect('button-press-event', self.on_button_press_event)
        self.connect('key-press-event', self.on_key_press_event)
        self.connect('motion-notify-event', self.on_motion_notify_event)
        # settings
        self.resize(self.app.win_width, self.app.win_height)
        self.headerbar.lr_button.props.active = self.app.left_to_right
        # show
        self.show_all()
        self.cursor_default = self.props.window.get_cursor()
        self.hb_height = self.headerbar.get_allocated_height()

    def set_uri(self, uri):
        if uri == None:
            return
        if self.archive.new_archive(uri, self.app):
            title = GLib.path_get_basename(self.archive.path)
            self.headerbar.set_title(title)
            self.upperbar.set_title(title)
            self.canvas.is_pdf = self.archive.status == 3
            self.num = 0
            self.change_pixbuf(0)
            # thumbnail
            self.headerbar.grid_button.set_sensitive(False)
            self.thumbnail_array = []
            self.generator = self.create_thumbnail()
            GLib.idle_add(self.idle_function)
        else:
            path = GLib.filename_from_uri(uri)[0]
            ComipoliMessage.show(self.app.props.active_window, f'i can not open\n<i>{path}</i>')

    def idle_function(self):
        try:
            next(self.generator)
            return True
        except StopIteration:
            self.headerbar.grid_button.set_sensitive(True)
            return False
        except Exception as e:
            ComipoliMessage.show(self.app.props.active_window, e)
            return False

    def create_thumbnail(self):
        for buf in self.archive:
            if self.canvas.is_pdf:
                w, h = buf.get_size()
            else:
                w = buf.get_width()
                h = buf.get_height()
            if w > h:
                aw = self.app.thumbnail_height
                ah = round(self.app.thumbnail_height * h / w)
            else:
                aw = round(self.app.thumbnail_height * w / h)
                ah = self.app.thumbnail_height
            if self.canvas.is_pdf:
                with cairo.ImageSurface(cairo.Format.ARGB32, aw, ah) as surface:
                    cr = cairo.Context(surface)
                    matrix = cairo.Matrix(aw/w, 0, 0, ah/h, 0, 0)
                    cr.transform(matrix)
                    cr.set_source_rgb(1, 1, 1)
                    cr.rectangle(0, 0, w, h)
                    cr.fill()
                    buf.render(cr)
                    pixbuf = Gdk.pixbuf_get_from_surface(surface, 0, 0, aw, ah)
            else:
                pixbuf = buf.scale_simple(aw, ah, GdkPixbuf.InterpType.TILES)
            self.thumbnail_array.append(pixbuf)
            yield 1

    def on_next_info(self, widget, res_id):
        if res_id == Gtk.ResponseType.OK:
            self.set_uri(widget.uri)
        widget.hide()

    def change_pixbuf(self, nex):
        length = len(self.archive)
        if length == 0:
            return
        try:
            num = self.num + nex if self.canvas.spread else self.num + nex - 1
            if length < num + 1 and nex != 50:
                path = GLib.path_get_dirname(self.archive.path)
                etc = GLib.path_get_basename(self.archive.path) # if (zip|rar|7z) ext
                exts = 'cbz'
                if self.app.is_pdf:
                    exts += '|pdf'
                if self.app.is_unrar:
                    exts += '|cbr'
                if self.app.is_7za:
                    exts += '|cb7'
                cbzs = comipoli_numsort.get_file_numsort_list(path, exts, etc)
                l = len(cbzs)
                if l == 0:
                    return
                else:
                    next_index = cbzs.index(etc) + 1
                    if l > next_index:
                        dlg = ComipolNextDialog(html.escape(cbzs[next_index]))
                        #self.get_group().add_window(dlg)
                        dlg.set_transient_for(self)
                        if next_index < 2:
                            dlg.set_response_sensitive(Gtk.ResponseType.NO, False)
                            #elif next_index != 1:
                            #prev = html.escape(cbzs[next_index - 2])
                            #dlg.props.secondary_text = f'<span bgcolor="#9900CC">Prev:</span> {prev}'
                        res = dlg.run() 
                        if res == Gtk.ResponseType.YES:
                            uri = GLib.filename_to_uri(GLib.build_filenamev([path, cbzs[next_index]]), None)
                            self.set_uri(uri)
                        elif res == Gtk.ResponseType.NO:
                            uri = GLib.filename_to_uri(GLib.build_filenamev([path, cbzs[next_index - 2]]), None)
                            self.set_uri(uri)
                        dlg.destroy()
                    elif self.is_fullscreen:
                        self.toggle_fullscreen()
            else:
                if nex == 0:
                    # Start Page
                    self.canvas.first_page = self.archive[0]
                    self.num = 0
                    self.canvas.spread = False
                    self.canvas.queue_draw()
                elif nex == 50:
                    # Last Page
                    self.canvas.first_page = self.archive[length - 1]
                    self.num = length - 1
                    self.canvas.spread = False
                    self.canvas.queue_draw()
                elif nex > 0:
                    # Next Page
                    if length > num:
                        # Set First Page
                        self.canvas.first_page = self.archive[num]
                        if self.is_oblong(self.canvas.first_page):
                            self.canvas.spread = False
                        else:
                            if length > num + 1:
                                # Spread View
                                self.canvas.second_page = self.archive[num + 1]
                                if self.is_oblong(self.canvas.second_page):
                                    self.canvas.spread = False
                                else:
                                    self.canvas.spread = True
                            else:
                                self.canvas.spread = False
                        self.num = num
                        self.canvas.queue_draw()
                else:
                    # Previos Page
                    num2 = self.num + nex
                    if num2 < 0:
                        self.num = 0
                        self.canvas.first_page = self.archive[0]
                        self.canvas.spread = False
                        self.canvas.queue_draw()
                    else:
                        # Check (width > height)
                        pixbuf = self.archive[num2]
                        pixbuf2 = self.archive[num2 + 1] if length > 1 else None
                        if self.is_oblong(pixbuf) or self.is_oblong(pixbuf2):
                            # Single View
                            self.canvas.first_page = pixbuf2
                            self.canvas.spread = False
                            self.num = num2 + 1
                        else:
                            # Spread View
                            self.canvas.first_page = pixbuf
                            self.canvas.second_page = pixbuf2
                            self.canvas.spread = True
                            self.num = num2
                        self.canvas.queue_draw()
                # Draw Page Number
                page_n = f'{self.num + 1}/{length}'
                self.headerbar.set_subtitle(page_n)
                self.upperbar.set_subtitle(page_n)
        except Exception as e:
            ComipoliMessage.show(self.app.props.active_window, e)

    def is_oblong(self, buf):
        if buf == None:
            return True
        if self.archive.status == 3:
            w, h = buf.get_size()
            return w - h > 0
        else:
            return buf.get_width() - buf.get_height() > 0

    def on_lr_button_toggled(self, button):
        if button.props.active:
            self.headerbar.lr_button.set_label('L->R')
            self.upperbar.lr_button.set_label('L->R')
            self.canvas.l_to_r = True
        else:
            self.headerbar.lr_button.set_label('L<-R')
            self.upperbar.lr_button.set_label('L<-R')
            self.canvas.l_to_r = False
        self.canvas.queue_draw()

    def on_motion_notify_event(self, widget, event):
        if len(self.archive) == 0:
            return
        width = self.get_size()[0]
        if self.is_fullscreen:
            if self.menuf.popover.props.visible:
                return
            if event.y <= self.hb_height and self.upperbar.props.visible == False:
                self.upperbar.show()
            elif event.y > self.hb_height and self.upperbar.props.visible:
                self.upperbar.hide()
        if event.x < self.app.cursor_change_pixel:
            if self.cursor_id != 1:
                self.canvas.props.window.set_cursor(self.cursor_left)
                self.cursor_id = 1
        elif width - self.app.cursor_change_pixel < event.x:
            if self.cursor_id != 2:
                self.canvas.props.window.set_cursor(self.cursor_right)
                self.cursor_id = 2
        else:
            if self.is_fullscreen:
                if self.cursor_id != 3:
                    self.canvas.props.window.set_cursor(self.cursor_blank)
                    self.cursor_id = 3
            else:
                if self.cursor_id != 0:
                    self.canvas.props.window.set_cursor(self.cursor_default)
                    self.cursor_id = 0

    def on_grid_button_clicked(self, button):
        dlg = ComipoliThumbnail(self)
        dlg.run()
        dlg.destroy()

    def open_dialog(self):
        dlg = Gtk.FileChooserNative(
            title='Open',
            transient_for=self,
            action=Gtk.FileChooserAction.OPEN)
        if len(self.archive) > 0:
            d = GLib.path_get_dirname(self.archive.path)
            dlg.set_current_folder(d)
        ft = Gtk.FileFilter()
        ft.set_name('Comic Book Archive')
        ft.add_mime_type('application/x-cbz')
        ft.add_mime_type('application/vnd.comicbook+zip')
        if self.app.is_pdf:
            ft.add_mime_type('application/pdf')
        if self.app.is_unrar:
            ft.add_mime_type('application/x-cbr')
        if self.app.is_7za:
            ft.add_mime_type('application/x-cb7')
        dlg.add_filter(ft)
        if dlg.run() == Gtk.ResponseType.ACCEPT:
            self.set_uri(dlg.get_uri())
        dlg.destroy()

    def on_key_press_event(self, widget, event):
        '''
            EoG Like Key Bind
        '''
        # Cancel Long Keypress
        if self.long_keypress != event.time:
            self.long_keypress = event.time
            self.key_press(event.state, event.keyval)
        return False

    def key_press(self, state, keyval):
        # CapsLock?
        keymap = Gdk.Keymap.get_default()
        _state = state ^ Gdk.ModifierType.LOCK_MASK if keymap.get_caps_lock_state() else state
        # UpperCase?
        val = Gdk.keyval_to_lower(keyval) if Gdk.keyval_is_upper(keyval) else keyval
        #
        if _state == Gdk.ModifierType.CONTROL_MASK:
            # Step Page
            if val == Gdk.KEY_space:
                self.change_pixbuf(1)
            elif val == Gdk.KEY_BackSpace:
                self.change_pixbuf(-1)
            # Preference
            elif val == Gdk.KEY_Right:
                if self.headerbar.lr_button.props.active:
                    self.change_pixbuf(1)   
                else:
                    self.change_pixbuf(-1)
            elif val == Gdk.KEY_Left:
                if self.headerbar.lr_button.props.active:
                    self.change_pixbuf(-1)   
                else:
                    self.change_pixbuf(1)
            # Close
            elif val == Gdk.KEY_w:
                self.close()
        else:
            # Next Page
            if val == Gdk.KEY_space:
                self.change_pixbuf(2)
            elif val == Gdk.KEY_BackSpace:
                self.change_pixbuf(-2)
            # Preference
            elif val == Gdk.KEY_Right:
                if self.headerbar.lr_button.props.active:
                    self.change_pixbuf(2)   
                else:
                    self.change_pixbuf(-2)
            elif val == Gdk.KEY_Left:
                if self.headerbar.lr_button.props.active:
                    self.change_pixbuf(-2)   
                else:
                    self.change_pixbuf(2)
            # Home, End
            elif val == Gdk.KEY_Home:
                self.change_pixbuf(0)
            elif val == Gdk.KEY_End:
                self.change_pixbuf(50)
            # etc
            elif val == Gdk.KEY_F10:
                if self.is_fullscreen:
                    self.upperbar.show()
                    GLib.idle_add(self.show_fullscreen_menu)
                else:
                    self.menu.clicked()
            elif val == Gdk.KEY_F11:
                self.toggle_fullscreen()
            elif val == Gdk.KEY_Escape:
                if self.upperbar.props.visible:
                    self.upperbar.hide()
                elif self.is_fullscreen:
                    self.toggle_fullscreen()
                elif self.app.esc_exit:
                    self.close()
            elif val == Gdk.KEY_F9:
                if self.headerbar.grid_button.props.sensitive:
                    self.headerbar.grid_button.clicked()
            elif val == Gdk.KEY_F8:
                self.headerbar.lr_button.props.active = not self.headerbar.lr_button.props.active
        return False

    def show_fullscreen_menu(self):
        self.menuf.clicked()
        return False

    def toggle_fullscreen(self):
        if self.is_fullscreen:
            self.unfullscreen()
            self.is_fullscreen = False
            self.canvas.props.window.set_cursor(self.cursor_default)
            self.cursor_id = 0
            if self.upperbar.props.visible:
                self.upperbar.hide()
        else:
            self.fullscreen()
            self.is_fullscreen = True
            self.canvas.props.window.set_cursor(self.cursor_blank)
            self.cursor_id = 3

    def on_restore_button_clicked(self, widget):
        self.toggle_fullscreen()

    def on_button_press_event(self, widget, event):
        if event.type == Gdk.EventType.DOUBLE_BUTTON_PRESS:
            if self.cursor_id != 1 and self.cursor_id != 2:
                self.toggle_fullscreen()
        elif self.long_click != event.time:
            self.longclick = event.time
            if self.cursor_id == 1:
                self.key_press(event.state, Gdk.KEY_Left)
            elif self.cursor_id == 2:
                self.key_press(event.state, Gdk.KEY_Right)
        return False

    def do_drag_data_received(self, drag_context, x, y, data, info, time):
        drop = data.get_uris()[0]
        self.set_uri(drop)
        self.present()

    def do_hide(self):
        settings = Gio.Settings(schema='org.sasakima.comipoli')
        xy = GLib.Variant('(ii)', self.get_size())
        settings.set_value('window-size', xy)
        settings.set_boolean('left-to-right', self.headerbar.lr_button.props.active)

